@@ -20,6 +20,8 @@ import copy
 import binascii
 import time
 
+import xml.dom.minidom
+
 try:
     from lxml import etree as ET
 except ImportError:
@@ -42,13 +44,36 @@ from libcloud.utils.py3 import StringIO
 from libcloud.utils.py3 import u
 from libcloud.utils.py3 import b
 
-from libcloud.utils.misc import lowercase_keys
+from libcloud.utils.misc import lowercase_keys, retry
 from libcloud.utils.compression import decompress_data
-from libcloud.common.types import LibcloudError, MalformedResponseError
 
+from libcloud.common.exceptions import exception_from_message
+from libcloud.common.types import LibcloudError, MalformedResponseError
+from libcloud.httplib_ssl import LibcloudHTTPConnection
 from libcloud.httplib_ssl import LibcloudHTTPSConnection
 
-LibcloudHTTPConnection = httplib.HTTPConnection
+__all__ = [
+    'RETRY_FAILED_HTTP_REQUESTS',
+
+    'BaseDriver',
+
+    'Connection',
+    'PollingConnection',
+    'ConnectionKey',
+    'ConnectionUserAndKey',
+    'CertificateConnection',
+    'LoggingHTTPConnection',
+    'LoggingHTTPSConnection',
+
+    'Response',
+    'HTTPResponse',
+    'JsonResponse',
+    'XmlResponse',
+    'RawResponse'
+]
+
+# Module level variable indicates if the failed HTTP requests should be retried
+RETRY_FAILED_HTTP_REQUESTS = False
 
 
 class HTTPResponse(httplib.HTTPResponse):
@@ -113,7 +138,9 @@ class Response(object):
             self.body = b(self.body).decode('utf-8')
 
         if not self.success():
-            raise Exception(self.parse_error())
+            raise exception_from_message(code=self.status,
+                                         message=self.parse_error(),
+                                         headers=self.headers)
 
         self.object = self.parse_body()
 
@@ -267,7 +294,9 @@ class LoggingConnection():
 
     :cvar log: file-like object that logs entries are written to.
     """
+
     log = None
+    http_proxy_used = False
 
     def _log_response(self, r):
         rv = "# -------- begin %d:%d response ----------\n" % (id(self), id(r))
@@ -284,7 +313,7 @@ class LoggingConnection():
         ht += "\r\n"
 
         # this is evil. laugh with me. ha arharhrhahahaha
-        class fakesock:
+        class fakesock(object):
             def __init__(self, s):
                 self.s = s
 
@@ -300,17 +329,36 @@ class LoggingConnection():
         headers = lowercase_keys(dict(r.getheaders()))
 
         encoding = headers.get('content-encoding', None)
+        content_type = headers.get('content-type', None)
 
         if encoding in ['zlib', 'deflate']:
             body = decompress_data('zlib', body)
         elif encoding in ['gzip', 'x-gzip']:
             body = decompress_data('gzip', body)
 
+        pretty_print = os.environ.get('LIBCLOUD_DEBUG_PRETTY_PRINT_RESPONSE',
+                                      False)
+
         if r.chunked:
             ht += "%x\r\n" % (len(body))
-            ht += u(body)
+            ht += body.decode('utf-8')
             ht += "\r\n0\r\n"
         else:
+            if pretty_print and content_type == 'application/json':
+                try:
+                    body = json.loads(body.decode('utf-8'))
+                    body = json.dumps(body, sort_keys=True, indent=4)
+                except:
+                    # Invalid JSON or server is lying about content-type
+                    pass
+            elif pretty_print and content_type == 'text/xml':
+                try:
+                    elem = xml.dom.minidom.parseString(body.decode('utf-8'))
+                    body = elem.toprettyxml()
+                except Exception:
+                    # Invalid XML
+                    pass
+
             ht += u(body)
 
         if sys.version_info >= (2, 6) and sys.version_info < (2, 7):
@@ -329,7 +377,21 @@ class LoggingConnection():
         return (rr, rv)
 
     def _log_curl(self, method, url, body, headers):
-        cmd = ["curl", "-i"]
+        cmd = ["curl"]
+
+        if self.http_proxy_used:
+            if self.proxy_username and self.proxy_password:
+                proxy_url = 'http://%s:%s@%s:%s' % (self.proxy_username,
+                                                    self.proxy_password,
+                                                    self.proxy_host,
+                                                    self.proxy_port)
+            else:
+                proxy_url = 'http://%s:%s' % (self.proxy_host,
+                                              self.proxy_port)
+            proxy_url = pquote(proxy_url)
+            cmd.extend(['--proxy', proxy_url])
+
+        cmd.extend(['-i'])
 
         if method.lower() == 'head':
             # HEAD method need special handling
@@ -340,6 +402,11 @@ class LoggingConnection():
         for h in headers:
             cmd.extend(["-H", pquote("%s: %s" % (h, headers[h]))])
 
+        cert_file = getattr(self, 'cert_file', None)
+
+        if cert_file:
+            cmd.extend(["--cert", pquote(cert_file)])
+
         # TODO: in python 2.6, body can be a file-like object.
         if body is not None and len(body) > 0:
             cmd.extend(["--data-binary", pquote(body)])
@@ -419,11 +486,13 @@ class Connection(object):
     driver = None
     action = None
     cache_busting = False
+    backoff = None
+    retry_delay = None
 
     allow_insecure = True
 
     def __init__(self, secure=True, host=None, port=None, url=None,
-                 timeout=None):
+                 timeout=None, proxy_url=None, retry_delay=None, backoff=None):
         self.secure = secure and 1 or 0
         self.ua = []
         self.context = {}
@@ -451,8 +520,22 @@ class Connection(object):
             (self.host, self.port, self.secure,
              self.request_path) = self._tuple_from_url(url)
 
-        if timeout:
-            self.timeout = timeout
+        self.timeout = timeout or self.timeout
+        self.retry_delay = retry_delay
+        self.backoff = backoff
+        self.proxy_url = proxy_url
+
+    def set_http_proxy(self, proxy_url):
+        """
+        Set a HTTP proxy which will be used with this connection.
+
+        :param proxy_url: Proxy URL (e.g. http://<hostname>:<port> without
+                          authentication and
+                          http://<username>:<password>@<hostname>:<port> for
+                          basic auth authentication information.
+        :type proxy_url: ``str``
+        """
+        self.proxy_url = proxy_url
 
     def set_context(self, context):
         if not isinstance(context, dict):
@@ -477,7 +560,7 @@ class Connection(object):
 
         if ":" in netloc:
             netloc, port = netloc.rsplit(":")
-            port = port
+            port = int(port)
 
         if not port:
             if scheme == "http":
@@ -486,10 +569,11 @@ class Connection(object):
                 port = 443
 
         host = netloc
+        port = int(port)
 
         return (host, port, secure, request_path)
 
-    def connect(self, host=None, port=None, base_url=None):
+    def connect(self, host=None, port=None, base_url=None, **kwargs):
         """
         Establish a connection with the API server.
 
@@ -515,13 +599,31 @@ class Connection(object):
             host = host or self.host
             port = port or self.port
 
-        kwargs = {'host': host, 'port': int(port)}
+        # Make sure port is an int
+        port = int(port)
+
+        if not hasattr(kwargs, 'host'):
+            kwargs.update({'host': host})
+
+        if not hasattr(kwargs, 'port'):
+            kwargs.update({'port': port})
+
+        if not hasattr(kwargs, 'key_file') and hasattr(self, 'key_file'):
+            kwargs.update({'key_file': self.key_file})
+
+        if not hasattr(kwargs, 'cert_file') and hasattr(self, 'cert_file'):
+            kwargs.update({'cert_file': self.cert_file})
+
+        #  kwargs = {'host': host, 'port': int(port)}
 
         # Timeout is only supported in Python 2.6 and later
         # http://docs.python.org/library/httplib.html#httplib.HTTPConnection
         if self.timeout and not PY25:
             kwargs.update({'timeout': self.timeout})
 
+        if self.proxy_url:
+            kwargs.update({'proxy_url': self.proxy_url})
+
         connection = self.conn_classes[secure](**kwargs)
         # You can uncoment this line, if you setup a reverse proxy server
         # which proxies to your endpoint, and lets you easily capture
@@ -601,6 +703,9 @@ class Connection(object):
         else:
             headers = copy.copy(headers)
 
+        retry_enabled = os.environ.get('LIBCLOUD_RETRY_FAILED_HTTP_REQUESTS',
+                                       False) or RETRY_FAILED_HTTP_REQUESTS
+
         action = self.morph_action_hook(action)
         self.action = action
         self.method = method
@@ -664,8 +769,17 @@ class Connection(object):
 
                 self.connection.endheaders()
             else:
-                self.connection.request(method=method, url=url, body=data,
-                                        headers=headers)
+                if retry_enabled:
+                    retry_request = retry(timeout=self.timeout,
+                                          retry_delay=self.retry_delay,
+                                          backoff=self.backoff)
+                    retry_request(self.connection.request)(method=method,
+                                                           url=url,
+                                                           body=data,
+                                                           headers=headers)
+                else:
+                    self.connection.request(method=method, url=url, body=data,
+                                            headers=headers)
         except ssl.SSLError:
             e = sys.exc_info()[1]
             self.reset_context()
@@ -799,7 +913,7 @@ class PollingConnection(Connection):
 
         :type context: ``dict``
         :param context: Context dictionary which is passed to the functions
-        which construct initial and poll URL.
+                        which construct initial and poll URL.
 
         :return: An :class:`Response` instance.
         :rtype: :class:`Response` instance
@@ -872,17 +986,40 @@ class ConnectionKey(Connection):
     Base connection class which accepts a single ``key`` argument.
     """
     def __init__(self, key, secure=True, host=None, port=None, url=None,
-                 timeout=None):
+                 timeout=None, proxy_url=None, backoff=None, retry_delay=None):
         """
         Initialize `user_id` and `key`; set `secure` to an ``int`` based on
         passed value.
         """
         super(ConnectionKey, self).__init__(secure=secure, host=host,
                                             port=port, url=url,
-                                            timeout=timeout)
+                                            timeout=timeout,
+                                            proxy_url=proxy_url,
+                                            backoff=backoff,
+                                            retry_delay=retry_delay)
         self.key = key
 
 
+class CertificateConnection(Connection):
+    """
+    Base connection class which accepts a single ``cert_file`` argument.
+    """
+    def __init__(self, cert_file, secure=True, host=None, port=None, url=None,
+                 proxy_url=None, timeout=None, backoff=None, retry_delay=None):
+        """
+        Initialize `cert_file`; set `secure` to an ``int`` based on
+        passed value.
+        """
+        super(CertificateConnection, self).__init__(secure=secure, host=host,
+                                                    port=port, url=url,
+                                                    timeout=timeout,
+                                                    backoff=backoff,
+                                                    retry_delay=retry_delay,
+                                                    proxy_url=proxy_url)
+
+        self.cert_file = cert_file
+
+
 class ConnectionUserAndKey(ConnectionKey):
     """
     Base connection class which accepts a ``user_id`` and ``key`` argument.
@@ -890,11 +1027,15 @@ class ConnectionUserAndKey(ConnectionKey):
 
     user_id = None
 
-    def __init__(self, user_id, key, secure=True,
-                 host=None, port=None, url=None, timeout=None):
+    def __init__(self, user_id, key, secure=True, host=None, port=None,
+                 url=None, timeout=None, proxy_url=None,
+                 backoff=None, retry_delay=None):
         super(ConnectionUserAndKey, self).__init__(key, secure=secure,
                                                    host=host, port=port,
-                                                   url=url, timeout=timeout)
+                                                   url=url, timeout=timeout,
+                                                   backoff=backoff,
+                                                   retry_delay=retry_delay,
+                                                   proxy_url=proxy_url)
         self.user_id = user_id
 
 
@@ -955,6 +1096,10 @@ class BaseDriver(object):
         self.region = region
 
         conn_kwargs = self._ex_connection_class_kwargs()
+        conn_kwargs.update({'timeout': kwargs.pop('timeout', None),
+                            'retry_delay': kwargs.pop('retry_delay', None),
+                            'backoff': kwargs.pop('backoff', None),
+                            'proxy_url': kwargs.pop('proxy_url', None)})
         self.connection = self.connectionCls(*args, **conn_kwargs)
 
         self.connection.driver = self
