@@ -21,6 +21,7 @@ __all__ = [
     'abiquo',
     'brightbox',
     'bluebox',
+    'dimensiondata',
     'dummy',
     'ec2',
     'ecp',
@@ -39,4 +40,5 @@ __all__ = [
     'vcloud',
     'voxel',
     'vpsnet',
+    'onapp',
 ]
