@@ -19,6 +19,7 @@ import os
 import sys
 import unittest
 import datetime
+from libcloud.utils.iso8601 import UTC
 
 try:
     import simplejson as json
@@ -33,7 +34,8 @@ from libcloud.utils.py3 import u
 
 from libcloud.common.types import InvalidCredsError, MalformedResponseError, \
     LibcloudError
-from libcloud.compute.types import Provider, KeyPairDoesNotExistError
+from libcloud.compute.types import Provider, KeyPairDoesNotExistError, StorageVolumeState, \
+    VolumeSnapshotState
 from libcloud.compute.providers import get_driver
 from libcloud.compute.drivers.openstack import (
     OpenStack_1_0_NodeDriver, OpenStack_1_0_Response,
@@ -236,6 +238,7 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEqual(node.extra.get('flavorId'), '1')
         self.assertEqual(node.extra.get('imageId'), '11')
         self.assertEqual(type(node.extra.get('metadata')), type(dict()))
+
         OpenStackMockHttp.type = 'METADATA'
         ret = self.driver.list_nodes()
         self.assertEqual(len(ret), 1)
@@ -685,7 +688,7 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
 
         # assert that we use the base url and not the auth url
         self.assertEqual(self.driver.connection.host, 'ex_force_base_url.com')
-        self.assertEqual(self.driver.connection.port, '666')
+        self.assertEqual(self.driver.connection.port, 666)
         self.assertEqual(self.driver.connection.request_path, '/forced_url')
 
     def test_get_endpoint_populates_host_port_and_request_path(self):
@@ -697,7 +700,7 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
 
         # assert that we use the result of get endpoint
         self.assertEqual(self.driver.connection.host, 'endpoint_auth_url.com')
-        self.assertEqual(self.driver.connection.port, '1555')
+        self.assertEqual(self.driver.connection.port, 1555)
         self.assertEqual(self.driver.connection.request_path, '/service_url')
 
     def test_set_auth_token_populates_host_port_and_request_path(self):
@@ -710,7 +713,7 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         # assert that we use the base url and not the auth url
         self.assertEqual(
             self.driver.connection.host, 'some_other_ex_force_base_url.com')
-        self.assertEqual(self.driver.connection.port, '1222')
+        self.assertEqual(self.driver.connection.port, 1222)
         self.assertEqual(self.driver.connection.request_path, '/some-service')
 
     def test_auth_token_without_base_url_raises_exception(self):
@@ -777,6 +780,18 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEqual(node.extra.get('metadata'), {})
         self.assertEqual(node.extra['updated'], '2011-10-11T00:50:04Z')
         self.assertEqual(node.extra['created'], '2011-10-11T00:51:39Z')
+        self.assertEqual(node.extra.get('userId'), 'rs-reach')
+        self.assertEqual(node.extra.get('hostId'), '912566d83a13fbb357ea'
+                                                   '3f13c629363d9f7e1ba3f'
+                                                   '925b49f3d2ab725')
+        self.assertEqual(node.extra.get('disk_config'), 'AUTO')
+        self.assertEqual(node.extra.get('task_state'), 'spawning')
+        self.assertEqual(node.extra.get('vm_state'), 'active')
+        self.assertEqual(node.extra.get('power_state'), 1)
+        self.assertEqual(node.extra.get('progress'), 25)
+        self.assertEqual(node.extra.get('fault')['id'], 1234)
+        self.assertTrue(node.extra.get('service_name') is not None)
+        self.assertTrue(node.extra.get('uri') is not None)
 
     def test_list_nodes_no_image_id_attribute(self):
         # Regression test for LIBCLOD-455
@@ -793,6 +808,7 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
 
         self.assertEqual('cd76a3a1-c4ce-40f6-9b9f-07a61508938d', volume.id)
         self.assertEqual('test_volume_2', volume.name)
+        self.assertEqual(StorageVolumeState.AVAILABLE, volume.state)
         self.assertEqual(2, volume.size)
         self.assertEqual(volume.extra, {
             'description': '',
@@ -802,6 +818,7 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
                 "serverId": "12065",
                 "volumeId": "cd76a3a1-c4ce-40f6-9b9f-07a61508938d",
             }],
+            'snapshot_id': None,
             'state': 'available',
             'location': 'nova',
             'volume_type': 'None',
@@ -809,14 +826,17 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
             'created_at': '2013-06-24T11:20:13.000000',
         })
 
+        # also test that unknown state resolves to StorageVolumeState.UNKNOWN
         volume = volumes[1]
         self.assertEqual('cfcec3bc-b736-4db5-9535-4c24112691b5', volume.id)
         self.assertEqual('test_volume', volume.name)
         self.assertEqual(50, volume.size)
+        self.assertEqual(StorageVolumeState.UNKNOWN, volume.state)
         self.assertEqual(volume.extra, {
             'description': 'some description',
             'attachments': [],
-            'state': 'available',
+            'snapshot_id': '01f48111-7866-4cd2-986a-e92683c4a363',
+            'state': 'some-unknown-state',
             'location': 'nova',
             'volume_type': 'None',
             'metadata': {},
@@ -830,6 +850,21 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         for size in sizes:
             self.assertTrue(isinstance(size.price, float),
                             'Wrong size price type')
+            self.assertTrue(isinstance(size.ram, int))
+            self.assertTrue(isinstance(size.vcpus, int))
+            self.assertTrue(isinstance(size.disk, int))
+            self.assertTrue(isinstance(size.swap, int))
+            self.assertTrue(isinstance(size.ephemeral_disk, int) or
+                            size.ephemeral_disk is None)
+            self.assertTrue(isinstance(size.extra, dict))
+            if size.id == '1':
+                self.assertEqual(size.ephemeral_disk, 40)
+                self.assertEqual(size.extra, {
+                    "policy_class": "standard_flavor",
+                    "class": "standard1",
+                    "disk_io_index": "2",
+                    "number_of_data_disks": "0"
+                })
 
         self.assertEqual(sizes[0].vcpus, 8)
 
@@ -1395,8 +1430,38 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
             self.conn_classes[1].type = 'RACKSPACE'
 
         snapshots = self.driver.ex_list_snapshots()
-        self.assertEqual(len(snapshots), 2)
+        self.assertEqual(len(snapshots), 3)
+        self.assertEqual(snapshots[0].created, datetime.datetime(2012, 2, 29, 3, 50, 7, tzinfo=UTC))
+        self.assertEqual(snapshots[0].extra['created'], "2012-02-29T03:50:07Z")
         self.assertEqual(snapshots[0].extra['name'], 'snap-001')
+        self.assertEqual(snapshots[0].state, VolumeSnapshotState.AVAILABLE)
+
+        # invalid date is parsed as None
+        assert snapshots[2].created is None
+
+    def test_list_volume_snapshots(self):
+        volume = self.driver.list_volumes()[0]
+
+        # rackspace needs a different mocked response for snapshots, but not for volumes
+        if self.driver_type.type == 'rackspace':
+            self.conn_classes[0].type = 'RACKSPACE'
+            self.conn_classes[1].type = 'RACKSPACE'
+
+        snapshots = self.driver.list_volume_snapshots(volume)
+        self.assertEqual(len(snapshots), 1)
+        self.assertEqual(snapshots[0].id, '4fbbdccf-e058-6502-8844-6feeffdf4cb5')
+
+    def test_create_volume_snapshot(self):
+        volume = self.driver.list_volumes()[0]
+        if self.driver_type.type == 'rackspace':
+            self.conn_classes[0].type = 'RACKSPACE'
+            self.conn_classes[1].type = 'RACKSPACE'
+
+        ret = self.driver.create_volume_snapshot(volume,
+                                                 'Test Volume',
+                                                 ex_description='This is a test',
+                                                 ex_force=True)
+        self.assertEqual(ret.id, '3fbbcccf-d058-4502-8844-6feeffdf4cb5')
 
     def test_ex_create_snapshot(self):
         volume = self.driver.list_volumes()[0]
@@ -1406,9 +1471,19 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
 
         ret = self.driver.ex_create_snapshot(volume,
                                              'Test Volume',
-                                             'This is a test')
+                                             description='This is a test',
+                                             force=True)
         self.assertEqual(ret.id, '3fbbcccf-d058-4502-8844-6feeffdf4cb5')
 
+    def test_destroy_volume_snapshot(self):
+        if self.driver_type.type == 'rackspace':
+            self.conn_classes[0].type = 'RACKSPACE'
+            self.conn_classes[1].type = 'RACKSPACE'
+
+        snapshot = self.driver.ex_list_snapshots()[0]
+        ret = self.driver.destroy_volume_snapshot(snapshot)
+        self.assertTrue(ret)
+
     def test_ex_delete_snapshot(self):
         if self.driver_type.type == 'rackspace':
             self.conn_classes[0].type = 'RACKSPACE'
